@@ -83,7 +83,7 @@ ml_prepare_dataframe <- function(x,
     }
   }
 
-  # assemble features vector and return
+  # assemble features vector
   transformed <- ft_vector_assembler(df, features, envir$features)
 
   # return as vanilla spark dataframe
@@ -100,8 +100,7 @@ predict.ml_model <- function(object,
                              ...)
 {
   # 'sdf_predict()' does not necessarily return a data set with the same row
-  # order as the input data; generate a unique id and re-join the generated
-  # spark dataframe to ensure the row order is maintained
+  # order as the input data; generate a unique id and re-order based on this
   id <- random_string("id_")
   sdf <- newdata %>%
     sdf_with_unique_id(id) %>%
@@ -111,12 +110,13 @@ predict.ml_model <- function(object,
   params <- object$model.parameters
   predicted <- sdf_predict(object, sdf, ...)
 
-  # join prediction column on original data, then read prediction column
-  column <- sdf %>%
-    invoke("join", spark_dataframe(predicted), as.list(id)) %>%
-    sdf_read_column("prediction")
+  # re-order based on id column
+  arranged <- arrange_(predicted, .dots = as.list(id))
 
-  # re-order based on id
+  # read column
+  column <- sdf_read_column(arranged, "prediction")
+
+  # re-map label ids back to actual labels
   if (is.character(params$labels) && is.numeric(column))
     column <- params$labels[column + 1]
 
